@@ -7,8 +7,8 @@
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
 import abc
+import datetime
 import inspect
-import time
 import warnings
 
 import numpy
@@ -98,12 +98,12 @@ class BaseCarton(metaclass=abc.ABCMeta):
         assert self.program, 'carton subclass must override program'
 
         self.plan = targeting_plan
-        self.tag = __version__
+        self.tag = __version__ if self.plan != '0.0.0-test' else None
 
         if config_file:
             this_config = read_yaml_file(config_file)
         else:
-            this_config = config
+            this_config = config.copy()
 
         if self.plan not in this_config:
             raise TargetSelectionError(
@@ -134,6 +134,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         self.schema = schema or self.config.get('schema', None) or 'sandbox'
         self.table_name = table_name or f'temp_{self.name}'.replace('-', '_')
+        self.RModel = None
 
         if self.cadence:
             ncad = tdb.Cadence.select().where(tdb.Cadence.label == self.cadence).count()
@@ -141,6 +142,14 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         self.log = log
         self.has_run = False
+        self._disable_query_log = False
+
+        # We cannot set temp_buffers multiple times if there are temporary tables
+        # (as in add_optical_magnitudes) so we set it here.
+        if ('database_options' in self.config and
+                'temp_buffers' in self.config['database_options']):
+            temp_buffers = self.config['database_options'].pop('temp_buffers')
+            self.database.execute_sql(f'SET temp_buffers = \'{temp_buffers}\'')
 
     @property
     def path(self):
@@ -186,15 +195,13 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
             catalogid = peewee.BigIntegerField(primary_key=True)
             selected = peewee.BooleanField()
-            cadence = peewee.TextField(null=True)
-            priority = peewee.IntegerField()
 
             class Meta:
                 database = self.database
                 table_name = self.table_name
                 schema = self.schema
                 reflection_options = {'skip_foreign_keys': True,
-                                      'use_peewee_reflection': True}
+                                      'use_peewee_reflection': False}
                 use_reflection = True
 
         if not Model.table_exists():
@@ -247,7 +254,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
                 raise RuntimeError(f'Temporary table {path!r} already exists.')
 
         log.info('Running query ...')
-        version_id = cdb.Version.get(plan=self.xmatch_plan).id
+        version_id = self.get_version_id()
 
         # If build_query accepts a query_region parameter, call with the query
         # region. Otherwise will add the radial query condition later.
@@ -284,12 +291,15 @@ class BaseCarton(metaclass=abc.ABCMeta):
         if limit:
             query = query.limit(limit)
 
-        query_sql, params = query.sql()
+        query_sql, params = self.database.get_sql_context().sql(query).query()
         cursor = self.database.cursor()
         query_str = cursor.mogrify(query_sql, params).decode()
 
-        log.debug(color_text(f'CREATE TABLE IF NOT EXISTS {path} AS ' + query_str,
-                             'darkgrey'))
+        if not self._disable_query_log:
+            log.debug(color_text(f'CREATE TABLE IF NOT EXISTS {path} AS ' + query_str,
+                                 'darkgrey'))
+        else:
+            log.debug('Not printing VERY long query.')
 
         with self.database.atomic():
             with Timer() as timer:
@@ -299,6 +309,8 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         log.info(f'Created table {path!r} in {timer.interval:.3f} s.')
 
+        self.RModel = self.get_model()
+
         log.debug('Adding columns and indexes.')
 
         columns = [
@@ -307,34 +319,228 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         if 'selected' not in columns:
             execute_sql(f'ALTER TABLE {path} ADD COLUMN selected BOOL DEFAULT TRUE;')
+            self.RModel._meta.add_field('selected', peewee.BooleanField())
 
-        if 'cadence' not in columns:
+        if 'cadence' not in columns and self.cadence is None:
             execute_sql(f'ALTER TABLE {path} ADD COLUMN cadence VARCHAR;')
+            self.RModel._meta.add_field('cadence', peewee.TextField())
 
-        if 'priority' not in columns:
+        if 'priority' not in columns and self.priority is None:
             execute_sql(f'ALTER TABLE {path} ADD COLUMN priority INTEGER;')
+            self.RModel._meta.add_field('priority', peewee.IntegerField())
 
-        if 'instrument' not in columns:
+        if 'instrument' not in columns and self.instrument is None:
             execute_sql(f'ALTER TABLE {path} ADD COLUMN instrument TEXT;')
+            self.RModel._meta.add_field('instrument', peewee.TextField())
 
         execute_sql(f'ALTER TABLE {path} ADD PRIMARY KEY (catalogid);')
         execute_sql(f'CREATE INDEX ON {path} (selected);')
         execute_sql(f'ANALYZE {path};')
 
-        time.sleep(2)
-        ResultsModel = self.get_model()
-
-        n_rows = ResultsModel.select().count()
+        n_rows = self.RModel.select().count()
         log.debug(f'Table {path!r} contains {n_rows:,} rows.')
 
         log.debug('Running post-process.')
         with self.database.atomic():
             self.setup_transaction()
-            self.post_process(ResultsModel, **post_process_kawrgs)
+            self.post_process(self.RModel, **post_process_kawrgs)
+
+        n_selected = self.RModel.select().where(self.RModel.selected >> True).count()
+        log.debug(f'Selected {n_selected:,} rows after post-processing.')
+
+        log.debug('Adding optical magnitude columns.')
+        self.add_optical_magnitudes()
 
         self.has_run = True
 
-        return ResultsModel
+        return self.RModel
+
+    def get_version_id(self):
+        """Returns the version_id for the cross-match plan."""
+
+        return cdb.Version.get(plan=self.xmatch_plan).id
+
+    def add_optical_magnitudes(self):
+        """Adds ``gri`` magnitude columns."""
+
+        Model = self.RModel
+
+        magnitudes = ['g', 'r', 'i', 'z']
+
+        # Check if ALL the columns have already been created in the query.
+        # If so, just return.
+        if any([mag in Model._meta.columns for mag in magnitudes]):
+            if not all([mag in Model._meta.columns for mag in magnitudes]):
+                raise TargetSelectionError(
+                    'Some optical magnitudes are defined in the query '
+                    'but not all of them.')
+            if 'optical_prov' not in Model._meta.columns:
+                raise TargetSelectionError('optical_prov column does not exist.')
+            warnings.warn('All optical magnitude columns are defined in the query.',
+                          TargetSelectionUserWarning)
+            return
+
+        # First create the columns. Also create z to speed things up. We won't
+        # use transformations for z but we can use the initial query to populate
+        # it and avoid doing the same query later when loading the magnitudes.
+        for mag in magnitudes:
+            self.database.execute_sql(f'ALTER TABLE {self.path} ADD COLUMN {mag} REAL;')
+            Model._meta.add_field(mag, peewee.FloatField())
+
+        self.database.execute_sql(f'ALTER TABLE {self.path} ADD COLUMN optical_prov TEXT;')
+        Model._meta.add_field('optical_prov', peewee.TextField())
+
+        # Step 1: join with sdss_dr13_photoobj and use SDSS magnitudes.
+
+        with self.database.atomic():
+
+            self.database.execute_sql('DROP TABLE IF EXISTS ' + self.table_name + '_sdss')
+            temp_table = peewee.Table(self.table_name + '_sdss')
+
+            (Model
+             .select(Model.catalogid,
+                     cdb.SDSS_DR13_PhotoObj.psfmag_g,
+                     cdb.SDSS_DR13_PhotoObj.psfmag_r,
+                     cdb.SDSS_DR13_PhotoObj.psfmag_i,
+                     cdb.SDSS_DR13_PhotoObj.psfmag_z)
+             .join(cdb.CatalogToSDSS_DR13_PhotoObj_Primary,
+                   on=(cdb.CatalogToSDSS_DR13_PhotoObj_Primary.catalogid == Model.catalogid))
+             .join(cdb.SDSS_DR13_PhotoObj,
+                   on=(cdb.CatalogToSDSS_DR13_PhotoObj_Primary.target_id ==
+                       cdb.SDSS_DR13_PhotoObj.objid))
+             .where(cdb.CatalogToSDSS_DR13_PhotoObj_Primary.best >> True,
+                    cdb.CatalogToSDSS_DR13_PhotoObj_Primary.version_id == self.get_version_id())
+             .where(Model.selected >> True)
+             .create_table(temp_table._path[0], temporary=True))
+
+            nrows = (Model.update(
+                {Model.g: temp_table.c.psfmag_g,
+                 Model.r: temp_table.c.psfmag_r,
+                 Model.i: temp_table.c.psfmag_i,
+                 Model.optical_prov: peewee.Value('sdss_psfmag')})
+                .from_(temp_table)
+                .where(Model.catalogid == temp_table.c.catalogid)
+                .where(temp_table.c.psfmag_g.is_null(False) &
+                       temp_table.c.psfmag_r.is_null(False) &
+                       temp_table.c.psfmag_i.is_null(False))).execute()
+
+        self.log.debug(f'{nrows:,} associated with SDSS magnitudes.')
+
+        # Step 2: localise entries with empty magnitudes and use PanSTARRS1
+        # transformations.
+
+        # PS1 fluxes are in Janskys. We use stacked fluxes instead of mean
+        # magnitudes since they are more complete on the faint end.
+        ps1_g = 8.9 - 2.5 * peewee.fn.log(cdb.Panstarrs1.g_stk_psf_flux)
+        ps1_r = 8.9 - 2.5 * peewee.fn.log(cdb.Panstarrs1.r_stk_psf_flux)
+        ps1_i = 8.9 - 2.5 * peewee.fn.log(cdb.Panstarrs1.i_stk_psf_flux)
+
+        # Use transformations to SDSS from Tonry et al. 2012 (section 3.2, table 6).
+        x = ps1_g - ps1_r
+        ps1_sdss_g = 0.013 + 0.145 * x + 0.019 * x * x + ps1_g
+        ps1_sdss_r = -0.001 + 0.004 * x + 0.007 * x * x + ps1_r
+        ps1_sdss_i = -0.005 + 0.011 * x + 0.010 * x * x + ps1_i
+
+        with self.database.atomic():
+
+            self.database.execute_sql('DROP TABLE IF EXISTS ' + self.table_name + '_ps1')
+            temp_table = peewee.Table(self.table_name + '_ps1')
+
+            (Model
+             .select(Model.catalogid,
+                     ps1_sdss_g.alias('ps1_sdss_g'),
+                     ps1_sdss_r.alias('ps1_sdss_r'),
+                     ps1_sdss_i.alias('ps1_sdss_i'))
+             .join(cdb.CatalogToPanstarrs1,
+                   on=(cdb.CatalogToPanstarrs1.catalogid == Model.catalogid))
+             .join(cdb.Panstarrs1,
+                   on=(cdb.CatalogToPanstarrs1.target_id == cdb.Panstarrs1.catid_objid))
+             .where(Model.g.is_null() | Model.r.is_null() | Model.i.is_null())
+             .where(Model.selected >> True)
+             .where(cdb.CatalogToPanstarrs1.best >> True,
+                    cdb.CatalogToPanstarrs1.version_id == self.get_version_id())
+             .where(cdb.Panstarrs1.g_stk_psf_flux.is_null(False) &
+                    (cdb.Panstarrs1.g_stk_psf_flux > 0))
+             .where(cdb.Panstarrs1.r_stk_psf_flux.is_null(False) &
+                    (cdb.Panstarrs1.r_stk_psf_flux > 0))
+             .where(cdb.Panstarrs1.i_stk_psf_flux.is_null(False) &
+                    (cdb.Panstarrs1.i_stk_psf_flux > 0))
+             .create_table(temp_table._path[0], temporary=True))
+
+            nrows = (Model.update(
+                {Model.g: temp_table.c.ps1_sdss_g,
+                 Model.r: temp_table.c.ps1_sdss_r,
+                 Model.i: temp_table.c.ps1_sdss_i,
+                 Model.optical_prov: peewee.Value('sdss_psfmag_ps1')})
+                .from_(temp_table)
+                .where(Model.catalogid == temp_table.c.catalogid)
+                .where(temp_table.c.ps1_sdss_g.is_null(False) &
+                       temp_table.c.ps1_sdss_r.is_null(False) &
+                       temp_table.c.ps1_sdss_i.is_null(False))).execute()
+
+        self.log.debug(f'{nrows:,} associated with PS1 magnitudes.')
+
+        # Step 3: localise entries with empty magnitudes and use Gaia transformations
+        # from Evans et al (2018).
+
+        gaia_G = cdb.Gaia_DR2.phot_g_mean_mag
+        gaia_BP = cdb.Gaia_DR2.phot_bp_mean_mag
+        gaia_RP = cdb.Gaia_DR2.phot_rp_mean_mag
+
+        x = gaia_BP - gaia_RP
+        x2 = x * x
+        x3 = x * x * x
+        gaia_sdss_g = -1 * (0.13518 - 0.46245 * x - 0.25171 * x2 + 0.021349 * x3) + gaia_G
+        gaia_sdss_r = -1 * (-0.12879 + 0.24662 * x - 0.027464 * x2 - 0.049465 * x3) + gaia_G
+        gaia_sdss_i = -1 * (-0.29676 + 0.64728 * x - 0.10141 * x2) + gaia_G
+
+        with self.database.atomic():
+
+            self.database.execute_sql('DROP TABLE IF EXISTS ' + self.table_name + '_gaia')
+            temp_table = peewee.Table(self.table_name + '_gaia')
+
+            (Model
+             .select(Model.catalogid,
+                     gaia_sdss_g.alias('gaia_sdss_g'),
+                     gaia_sdss_r.alias('gaia_sdss_r'),
+                     gaia_sdss_i.alias('gaia_sdss_i'))
+             .join(cdb.CatalogToTIC_v8, on=(cdb.CatalogToTIC_v8.catalogid == Model.catalogid))
+             .join(cdb.TIC_v8)
+             .join(cdb.Gaia_DR2)
+             .where(Model.g.is_null() | Model.r.is_null() | Model.i.is_null())
+             .where(Model.selected >> True)
+             .where(cdb.CatalogToTIC_v8.best >> True,
+                    cdb.CatalogToTIC_v8.version_id == self.get_version_id())
+             .where(cdb.Gaia_DR2.phot_g_mean_mag.is_null(False))
+             .where(cdb.Gaia_DR2.phot_bp_mean_mag.is_null(False))
+             .where(cdb.Gaia_DR2.phot_rp_mean_mag.is_null(False))
+             .create_table(temp_table._path[0], temporary=True))
+
+            nrows = (Model.update(
+                {Model.g: temp_table.c.gaia_sdss_g,
+                 Model.r: temp_table.c.gaia_sdss_r,
+                 Model.i: temp_table.c.gaia_sdss_i,
+                 Model.optical_prov: peewee.Value('sdss_psfmag_gaia')})
+                .from_(temp_table)
+                .where(Model.catalogid == temp_table.c.catalogid)
+                .where(temp_table.c.gaia_sdss_g.is_null(False) &
+                       temp_table.c.gaia_sdss_r.is_null(False) &
+                       temp_table.c.gaia_sdss_i.is_null(False))).execute()
+
+        self.log.debug(f'{nrows:,} associated with Gaia magnitudes.')
+
+        # Finally, check if there are any rows in which at least some of the
+        # magnitudes are null.
+
+        n_empty = (Model
+                   .select()
+                   .where(Model.g.is_null() | Model.r.is_null() | Model.i.is_null())
+                   .where(Model.selected >> True)
+                   .count())
+
+        if n_empty > 0:
+            warnings.warn(f'Found {n_empty} entries with empty magnitudes.',
+                          TargetSelectionUserWarning)
 
     def post_process(self, model, **kwargs):
         """Post-processes the temporary table.
@@ -422,10 +628,10 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         if mode == 'results':
 
-            results_model = self.get_model()
+            results_model = self.RModel
             assert results_model.table_exists(), 'temporary table does not exist.'
 
-            write_query = results_model.select()
+            write_query = results_model.select().order_by(results_model.catalogid)
 
             colnames = [field.name for field in write_query._returning]
 
@@ -454,6 +660,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
                     tdb.Version.plan == self.plan,
                     tdb.Version.target_selection >> True,
                 )
+                .order_by(tdb.Target.catalogid)
             )
 
             colnames = []
@@ -497,8 +704,8 @@ class BaseCarton(metaclass=abc.ABCMeta):
             if overwrite:
                 warnings.warn(
                     f'Carton {self.name!r} with plan {self.plan!r} '
-                    f'and tag {self.tag!r} already has targets '
-                    'loaded. Dropping carton-to-target entries.',
+                    f'already has targets loaded. '
+                    'Dropping carton-to-target entries.',
                     TargetSelectionUserWarning,
                 )
                 self.drop_carton()
@@ -506,11 +713,14 @@ class BaseCarton(metaclass=abc.ABCMeta):
                 raise TargetSelectionError(
                     f'Found existing targets for '
                     f'carton {self.name!r} with plan '
-                    f'{self.plan!r} and tag '
-                    f'{self.tag!r}.'
+                    f'{self.plan!r}.'
                 )
 
-        RModel = self.get_model()
+        if self.RModel is None:
+            RModel = self.get_model()
+        else:
+            RModel = self.RModel
+
         if not RModel.table_exists():
             raise TargetSelectionError(
                 f'No temporary table found {self.full}. Did you call run()?'
@@ -558,7 +768,10 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         # Create targeting plan in tdb.
         version, created = tdb.Version.get_or_create(
-            plan=self.plan, tag=self.tag, target_selection=True
+            plan=self.plan,
+            tag=self.tag,
+            target_selection=True,
+            robostrategy=False
         )
         version_pk = version.pk
 
@@ -594,6 +807,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
             program=self.program,
             mapper_pk=mapper_pk,
             version_pk=version_pk,
+            run_on=datetime.datetime.now().isoformat().split('T')[0]
         ).save()
 
         log.debug(f'Created carton {self.name!r}')
@@ -603,7 +817,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         log.debug('loading data into targetdb.target.')
 
-        n_inserted = (
+        n_inserted = (  # noqa: 841
             tdb.Target.insert_from(
                 cdb.Catalog.select(
                     cdb.Catalog.catalogid,
@@ -637,7 +851,7 @@ class BaseCarton(metaclass=abc.ABCMeta):
             .execute()
         )
 
-        log.info(f'Inserted {n_inserted:,} new rows into targetdb.target.')
+        log.info('Inserted new rows into targetdb.target.')
 
         return
 
@@ -668,9 +882,6 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
         for mag, mpath in magnitude_paths.items():
 
-            if mag == 'optical_prov':
-                continue
-
             fields.append(getattr(Magnitude, mag))
             if hasattr(RModel, mag):
                 select_from = select_from.select_extend(getattr(RModel, mag))
@@ -704,7 +915,9 @@ class BaseCarton(metaclass=abc.ABCMeta):
                             on=(tdb.Target.catalogid == node_model.catalogid),
                         )
                         select_from = select_from.where(
-                            (node_model.best >> True) | (node_model.catalogid >> None)
+                            ((node_model.best >> True) &
+                             (node_model.version_id == self.get_version_id())) |
+                            (node_model.catalogid >> None)
                         )
                     else:
                         select_from = select_from.join(
@@ -713,15 +926,20 @@ class BaseCarton(metaclass=abc.ABCMeta):
                 if column:
                     select_from = select_from.select_extend(getattr(node_model, column))
 
-        if 'optical_prov' in RModel._meta.columns:
-            select_from = select_from.select_extend(RModel._meta.columns['optical_prov'])
-        else:
-            select_from = select_from.select_extend(magnitude_paths['optical_prov'])
+        # Add gri from the temporary table.
+        for mag in ['g', 'r', 'i', 'z']:
+            select_from = select_from.select_extend(RModel._meta.columns[mag])
+            fields.append(Magnitude._meta.columns[mag])
+
+        if 'optical_prov' not in RModel._meta.columns:
+            raise TargetSelectionError('optical_prov column not found in temporary table.')
+
+        select_from = select_from.select_extend(RModel._meta.columns['optical_prov'])
         fields.append(Magnitude.optical_prov)
 
-        n_inserted = Magnitude.insert_from(select_from, fields).returning().execute()
+        n_inserted = Magnitude.insert_from(select_from, fields).returning().execute()  # noqa: 841
 
-        log.info(f'Inserted {n_inserted:,} new rows into targetdb.magnitude.')
+        log.info('Inserted new rows into targetdb.magnitude.')
 
     def _load_carton_to_target(self, RModel):
         """Populate targetdb.carton_to_target."""
@@ -759,19 +977,29 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
             # Check that not both the carton cadence and the cadence column
             # are not null.
-            if RModel.select().where(~(RModel.cadence >> None)).exists():
-                raise TargetSelectionError(
-                    'both carton cadence and target '
-                    'cadence defined. This is not '
-                    'allowed.'
-                )
+            if 'cadence' in RModel._meta.fields:
+                if RModel.select().where(~(RModel.cadence >> None)).exists():
+                    raise TargetSelectionError(
+                        'both carton cadence and target '
+                        'cadence defined. This is not '
+                        'allowed.'
+                    )
 
             cadence_pk = tdb.Cadence.get(label=self.cadence)
             select_from = select_from.select_extend(cadence_pk)
 
             if not self.value:
+                # improve robustness of cadence name pattern matching slightly:
+                try:
+                    cadence_payload = [s for s in
+                                       self.cadence.split("_")
+                                       if 'x' in s][0].split('x')
+                except BaseException:
+                    raise ("Uninterpretable cadence name: ", self.cadence)
+
                 self.value = float(numpy.multiply(
-                    *map(int, self.cadence.split('_')[-1].split('x'))
+                    # *map(int, self.cadence.split('_')[-1].split('x'))
+                    *map(int, cadence_payload)
                 ))
 
         else:
@@ -824,7 +1052,11 @@ class BaseCarton(metaclass=abc.ABCMeta):
 
                 values = tuple(
                     int(numpy.multiply(
-                        *map(int, cadence.split('_')[-1].split('x'))))
+                        # *map(int, cadence.split('_')[-1].split('x'))))
+                        # improve robustness of cadence name pattern matching slightly:
+                        *map(int, [s for s in
+                                   cadence.split("_")
+                                   if 'x' in s][0].split('x'))))
                     for cadence in data[:, 1]
                 )
 
@@ -850,16 +1082,31 @@ class BaseCarton(metaclass=abc.ABCMeta):
             select_from = select_from.select_extend(
                 tdb.Instrument.get(label=self.instrument).pk)
         else:
-            select_from = select_from.select_extend(RModel.instrument)
+            raise RuntimeError(f'Instrument not defined for carton {self.name}')
 
-        for colname in ['delta_ra', 'delta_dec', 'intertial']:
+        for colname in ['delta_ra', 'delta_dec', 'inertial']:
             if colname in RModel._meta.columns:
                 select_from = select_from.select_extend(RModel._meta.columns[colname])
             else:
-                select_from = select_from.select_extend(peewee.SQL('null'))
+                if colname == 'inertial':
+                    select_from = select_from.select_extend(peewee.Value(False))
+                else:
+                    select_from = select_from.select_extend(peewee.Value(0.0))
+
+        if 'lambda_eff' in RModel._meta.columns:
+            select_from = select_from.select_extend(RModel._meta.columns['lambda_eff'])
+        else:
+            if self.instrument is not None:
+                instrument = self.instrument
+            else:
+                instrument = RModel.instrument
+            select_from = select_from.select_extend(
+                tdb.Instrument
+                .select(tdb.Instrument.default_lambda_eff)
+                .where(tdb.Instrument.label == instrument))
 
         # Now do the insert
-        n_inserted = (
+        n_inserted = (  # noqa: 841
             CartonToTarget.insert_from(
                 select_from,
                 [
@@ -872,13 +1119,14 @@ class BaseCarton(metaclass=abc.ABCMeta):
                     CartonToTarget.delta_ra,
                     CartonToTarget.delta_dec,
                     CartonToTarget.inertial,
+                    CartonToTarget.lambda_eff
                 ],
             )
             .returning()
             .execute()
         )
 
-        log.info(f'Inserted {n_inserted:,} rows into targetdb.carton_to_target.')
+        log.info('Inserted rows into targetdb.carton_to_target.')
 
     def drop_carton(self):
         """Drops the entry in ``targetdb.carton``."""
