@@ -65,6 +65,9 @@ class MWM_CB_UVEX_BaseCarton(BaseCarton):
 
     def post_process(self, model, **kwargs):
 
+        if self.cadence is not None:
+            return
+
         self.database.execute_sql(f'ALTER TABLE {self.path} ADD COLUMN value REAL;')
         model._meta.add_field('value', peewee.FloatField())
 
@@ -72,15 +75,15 @@ class MWM_CB_UVEX_BaseCarton(BaseCarton):
         model.update(cadence='bright_1x1').where(model.phot_g_mean_mag < 17).execute()
         model.update(value=1).where(model.phot_g_mean_mag < 17).execute()
 
-        # 17 < G < 19 => cadence = dark_1x2
-        model.update(cadence='dark_1x2').where((model.phot_g_mean_mag > 17) &
+        # 17 <= G < 19 => cadence = dark_1x2
+        model.update(cadence='dark_1x2').where((model.phot_g_mean_mag >= 17) &
                                                (model.phot_g_mean_mag < 19)).execute()
-        model.update(value=2).where((model.phot_g_mean_mag > 17) &
+        model.update(value=2).where((model.phot_g_mean_mag >= 17) &
                                     (model.phot_g_mean_mag < 19)).execute()
 
-        # G > 19 => cadence = dark_1x3
-        model.update(cadence='dark_1x3').where(model.phot_g_mean_mag > 19).execute()
-        model.update(value=3).where(model.phot_g_mean_mag > 19).execute()
+        # G >= 19 => cadence = dark_1x3
+        model.update(cadence='dark_1x3').where(model.phot_g_mean_mag >= 19).execute()
+        model.update(value=3).where(model.phot_g_mean_mag >= 19).execute()
 
         return model
 
@@ -699,8 +702,8 @@ class MWM_CB_UVEX5_Carton(MWM_CB_UVEX_BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    instrument = 'BOSS'
-    cadence = None
+    instrument = 'APOGEE'
+    cadence = 'bright_3x1'
     priority = 1400
 
     def setup_transaction(self):
