@@ -63,7 +63,9 @@ class MWM_Halo_Best_Brightest_Base_Carton(BaseCarton):
                  .join(TIC_v8)
                  .join(Gaia_DR2)
                  .where(CatalogToAllWise.version_id == version_id,
-                        CatalogToAllWise.best >> True))
+                        CatalogToAllWise.best >> True,
+                        CatalogToTIC_v8.version_id == version_id,
+                        CatalogToTIC_v8.best >> True))
 
         if query_region:
             query = (query
@@ -77,24 +79,6 @@ class MWM_Halo_Best_Brightest_Base_Carton(BaseCarton):
         return query
 
 
-class MWM_Halo_Best_Brightest_APOGEE_Carton(MWM_Halo_Best_Brightest_Base_Carton):
-    """MWM Halo B&B targets for APOGEE."""
-
-    name = 'mwm_halo_bb_apogee'
-    mapper = 'MWM'
-    category = 'science'
-    program = 'mwm_filler'
-    instrument = 'APOGEE'
-    cadence = None
-    priority = 6050
-
-    def build_query(self, version_id, query_region=None):
-
-        query = super().build_query(version_id, query_region=query_region)
-
-        return query.where(BestBrightest.gmag < 13)
-
-
 class MWM_Halo_Best_Brightest_BOSS_Carton(MWM_Halo_Best_Brightest_Base_Carton):
     """MWM Halo B&B targets for BOSS."""
 
@@ -110,7 +94,7 @@ class MWM_Halo_Best_Brightest_BOSS_Carton(MWM_Halo_Best_Brightest_Base_Carton):
 
         query = super().build_query(version_id, query_region=query_region)
 
-        return query.where(BestBrightest.gmag >= 13)
+        return query
 
     def post_process(self, model, **kwargs):
 
@@ -180,6 +164,10 @@ class MWM_Halo_Best_Brightest_BOSS_Carton(MWM_Halo_Best_Brightest_Base_Carton):
             .where((parallax < 0) |
                    ~((abs_g < 5) | (parallax < 5 * parallax_error)))).execute()
 
+        # Assign a base priority to the remaining targets
+        (model.update({model.priority: 6090})
+            .where(model.priority.is_null())).execute()
+
 
 class MWM_Halo_SkyMapper_Base_Carton(BaseCarton):
     """MWM Halo SkyMapper.
@@ -219,24 +207,6 @@ class MWM_Halo_SkyMapper_Base_Carton(BaseCarton):
         return query
 
 
-class MWM_Halo_SkyMapper_APOGEE_Carton(MWM_Halo_SkyMapper_Base_Carton):
-    """MWM Halo SkyMapper APOGEE targets."""
-
-    name = 'mwm_halo_sm_apogee'
-    mapper = 'MWM'
-    category = 'science'
-    program = 'mwm_filler'
-    instrument = 'APOGEE'
-    cadence = 'bright_1x1'
-    priority = 6055
-
-    def build_query(self, version_id, **kwargs):
-
-        query = super().build_query(version_id, **kwargs)
-
-        return query.where(Gaia_DR2.phot_g_mean_mag < 13)
-
-
 class MWM_Halo_SkyMapper_BOSS_Carton(MWM_Halo_SkyMapper_Base_Carton):
     """MWM Halo SkyMapper BOSS targets."""
 
@@ -252,7 +222,7 @@ class MWM_Halo_SkyMapper_BOSS_Carton(MWM_Halo_SkyMapper_Base_Carton):
 
         query = super().build_query(version_id, **kwargs)
 
-        return query.where(Gaia_DR2.phot_g_mean_mag >= 13)
+        return query
 
     def post_process(self, model, **kwargs):
 
